import ddf.minim.*;
import ddf.minim.analysis.*;
import processing.io.*;

I2C IM920;
Minim minim;
AudioPlayer song;
BeatDetect beat;

int i2c_adr=0x30;
float eRadius1 = 20, eRadius2 = 20, eRadius3 = 20, eRadius4 = 20;

String filename = "";
int cancel = 0;

void setup() {
  size(400, 400);
  IM920 = new I2C(I2C.list()[0]);
  selectInput("Select a Sonund file:", "fileSelected");
  
  while(filename=="" && cancel==0) {delay(200);}

  if ( cancel == 0 ) {
    minim = new Minim(this);
    song = minim.loadFile(filename, 2048);
    song.play();

    beat = new BeatDetect(song.bufferSize(), song.sampleRate());
  
    ellipseMode(RADIUS);
  } else {
    exit();
  }
}

void fileSelected(File selection) {
  if (selection == null) {
    println("Window was closed or the user hit cancel.");
    cancel = 1;
  } else {
    filename = selection.getAbsolutePath();
    println("User selected " + filename);
  }
}

float left_max = 0.0, right_max = 0.0;

void draw() {
  background(0);
  beat.detect(song.mix);
  float a1 = map(eRadius1, 20, 80, 60, 255);
  fill(255, 255, 255, a1);
  byte onset_dat = 0;
  for(int i = 0; i < beat.detectSize(); ++i) {
    if ( beat.isOnset(i) ) {
      if ( onset_dat < 127 ) {
        onset_dat++;
      }
    }
  }
  if ( onset_dat > 0 ) {
    eRadius1 = 80;
  }
  ellipse(width/4, height/4, eRadius1, eRadius1);
  eRadius1 *= 0.95;
  if ( eRadius1 < 20 ) eRadius1 = 20;

  float a2 = map(eRadius2, 20, 80, 60, 255);
  fill(255, 0, 0, a2);
  if ( beat.isKick() ) eRadius2 = 80;
  ellipse(width/4*3, height/4, eRadius2, eRadius2);
  eRadius2 *= 0.95;
  if ( eRadius2 < 20 ) eRadius2 = 20;
  byte kick_dat = byte( int( map(eRadius2, 20, 80, 0, 255) ) );

  float a3 = map(eRadius3, 20, 80, 60, 255);
  fill(0, 255, 0, a3);
  if ( beat.isSnare() ) eRadius3 = 80;
  ellipse(width/4, height/4*3, eRadius3, eRadius3);
  eRadius3 *= 0.95;
  if ( eRadius3 < 20 ) eRadius3 = 20;
  byte snare_dat = byte( int( map(eRadius3, 20, 80, 0, 255) ) );

  float a4 = map(eRadius4, 20, 80, 60, 255);
  fill(0, 0, 255, a4);
  if ( beat.isHat() ) eRadius4 = 80;
  ellipse(width/4*3, height/4*3, eRadius4, eRadius4);
  eRadius4 *= 0.95;
  if ( eRadius4 < 20 ) eRadius4 = 20;
  byte hat_dat = byte( int( map(eRadius4, 20, 80, 0, 255) ) );

  fill( 255, 128 );
  
  rect( 0, 0, song.left.level()*width, 200 );
  rect( 0, 200, song.right.level()*width, 200 );
  
  if ( left_max < song.left.level() ) left_max = song.left.level();
  if ( right_max < song.right.level() ) right_max = song.right.level();
  
  text( song.left.level(), 0, 100);
  text( left_max, 0, 120);
  text( song.right.level(), 0, 300);
  text( right_max, 0, 320);
  
  float left = song.left.level();
  if ( left > 0.4 ) left = 0.4;
  byte left_val = byte( int( map( left, 0.0, 0.4, 0, 255 ) ) );
  
  float right = song.right.level();
  if ( right > 0.4 ) right = 0.4;
  byte right_val = byte( int( map( right, 0.0, 0.4, 0, 255 ) ) );
  
  String send_dat = "TXDA"+hex( onset_dat )+","+hex( kick_dat )+","+hex( snare_dat )+","+hex( hat_dat )+","+hex( left_val )+","+hex( right_val );
  text( send_dat, 0, 10);
  IM920.beginTransmission(i2c_adr);
  IM920.write(send_dat);
  IM920.endTransmission();
}
